@@ -1,7 +1,7 @@
 !abundances.f90, the core of NEAT which does all of the physical analysis
 !(C) Roger Wesson, Dave Stock, Peter Scicluna
-subroutine abundances(linelist,listlength,iteration_result,meanextinction,calculate_extinction,ILs,diagnostic_array,iion,atomicdata,maxlevs,maxtemps,heidata,switch_he,switch_icf,H_Balmer,H_Paschen,HeI_lines,HeII_lines,weights,subtract_recombination)
-use mod_abundtypes
+subroutine abundances(linelist,listlength,iteration_result,meanextinction,calculate_extinction,ILs,diagnostics,iion,atomicdata,maxlevs,maxtemps,heidata,switch_he,switch_icf,H_Balmer,H_Paschen,HeI_lines,HeII_lines,weights,subtract_recombination)
+use mod_types
 use mod_equib
 use mod_abundIO
 use mod_helium
@@ -11,16 +11,16 @@ use mod_resultarrays
 use mod_atomicdata
 use mod_oii_diagnostics
 use mod_hydrogen
+use mod_globals
 
         implicit none
-        integer, parameter :: dp=kind(1.d0)
         integer :: counter, i, j
         integer, intent(in) :: listlength
         type(line), dimension(listlength) :: linelist, linelist_orig
         character :: switch_he !switch for helium atomic data
         character :: switch_icf !switch for which ICF to use
         type(resultarray), dimension(1) :: iteration_result
-        real(kind=dp), dimension(6), intent(in) :: diagnostic_array
+        type(diagnostic_array), intent(in) :: diagnostics
         real(kind=dp) :: flux_no1, flux_no2, flux_no3, flux_no4, flux_no5, flux_no6
 
         real(kind=dp) :: oiiNratio, oiiDens, oiiiTratio, oiiiTemp, oiiiIRNratio, oiiiIRTratio, oiiiIRtemp, oiiiUVTratio, oiiiUVtemp, oiiiIRdens, niiTratio, niiTemp, ariiiIRNratio, ariiiIRdens, arivNratio, arivDens, cliiiNratio, cliiiDens, siiNratio, siiDens, siiTratio, siiTemp, siiiIRNratio, siiiIRdens, oiiTratio, oiiTemp, neiiiTratio, neiiiIRTratio, neiiiIRNratio, neiiiIRdens, neiiiTemp, neiiiIRTemp, oitemp, citemp
@@ -37,7 +37,8 @@ use mod_hydrogen
         real(kind=dp) :: heiabund,heiiabund,Hetotabund, heICFfactor, OICFfactor, upsilon, upsilonprime
         real(kind=dp) :: Te_balmer, Te_paschen
         real(kind=dp) :: oii4649, oii4089,oii4662,oii_te,oii_ne
-        real(kind=dp) :: ratio_5876_4471, ratio_6678_4471, te_5876_4471, te_6678_4471
+        real(kind=dp) :: ratio_5876_4471, ratio_6678_4471, ratio_7281_6678, te_5876_4471, te_6678_4471, te_7281_6678
+        real(kind=dp) :: aeff_hb,em_hb
 
         type(weightingarray) :: weights, weights_orig
 
@@ -76,10 +77,12 @@ use mod_hydrogen
 
         real(kind=dp) :: balmerdec_density, paschendec_density
 
+        logical :: oiiRLreliable,niiRLreliable ! report reliability of recombination line O/H and N/H abundances
+
 ! abundances relative to Halpha? todo, implement as command line option
 
         logical :: relativetoha=.false.
-        real, dimension(3) :: hafactor !factor for low, medium and high temperatures and densities
+        real(kind=dp), dimension(3) :: hafactor !factor for low, medium and high temperatures and densities
 
 ! strong line variables
         real(kind=dp) :: X23,O_R23upper, O_R23lower, N2,O_N2, O3N2, O_O3N2, Ar3O3, O_Ar3O3, S3O3, O_S3O3, x23temp1, x23temp2, x23temp3, x23temp4
@@ -171,7 +174,7 @@ use mod_hydrogen
             print "(4X,A9,F6.2)","H beta = ",(linelist(H_Balmer(4))%intensity)
           else
             print *,"   Specify the extinction from the command line with the -c option to proceed"
-            stop
+            call exit(201)
           endif
         endif
 
@@ -182,7 +185,7 @@ use mod_hydrogen
       do srloop=1,subtract_recombination
 
         if (calculate_extinction) then
-                call calc_extinction_coeffs(linelist,H_Balmer, c1, c2, c3, meanextinction, dble(10000.),dble(1000.),weights%ha,weights%hg,weights%hd)
+                call calc_extinction_coeffs(linelist,H_Balmer, c1, c2, c3, meanextinction, dble(10000.),dble(1000.),weights%ha,weights%hg,weights%hd,hidata)
 
                 if (meanextinction .lt. 0.0 .or. isnan(meanextinction)) then
                    meanextinction = 0.d0
@@ -286,9 +289,9 @@ use mod_hydrogen
 ! subtract recombination contributions assuming RL abundances
 ! on the first pass, rec. contrib. will still have its initial value of zero
 
-      if (nii5754recRL .gt. 0.d0) niiTratio = niiTratio / (1. - (0.01*nii5754recRL/get_cel_flux("nii5754    ",linelist,ILs)))
-      if (oii7325recRL .gt. 0.d0) oiiTratio = oiiTratio / (1. - (0.01*oii7325recRL/(get_cel_flux("oii7319b   ",linelist,ILs)+get_cel_flux("oii7330b   ",linelist,ILs))))
-      if (oiii4363recRL .gt. 0.d0) oiiiTratio = oiiiTratio / (1. - (0.01*oiii4363recRL/get_cel_flux("oiii4363   ",linelist,ILs)))
+      if (nii5754recRL .gt. 0.d0) niiTratio = niiTratio / (1. - (0.01*nii5754recRL))
+      if (oii7325recRL .gt. 0.d0) oiiTratio = oiiTratio / (1. - (0.01*oii7325recRL))
+      if (oiii4363recRL .gt. 0.d0) oiiiTratio = oiiiTratio / (1. - (0.01*oiii4363recRL))
 
 ! now get diagnostics zone by zone.
 
@@ -307,10 +310,10 @@ use mod_hydrogen
         if (oiidens .le. 0.d0) weights%oiiDens = 0.d0
         if (siidens .le. 0.d0) weights%siiDens = 0.d0
 
-        if ((weights%oiiDens + weights%siiDens) .eq. 0 .and. diagnostic_array(1) .eq. 0) then
+        if ((weights%oiiDens + weights%siiDens) .eq. 0 .and. diagnostics%lowdens .eq. 0) then
           lowdens = 1000.d0
-        elseif (diagnostic_array(1) .gt. 0.0) then
-          lowdens = diagnostic_array(1)
+        elseif (diagnostics%lowdens .gt. 0.0) then
+          lowdens = diagnostics%lowdens
         else
           lowdens = (weights%oiiDens*oiiDens + weights%siiDens*siiDens) / (weights%oiiDens + weights%siiDens)
         endif
@@ -329,10 +332,10 @@ use mod_hydrogen
         if (ciTemp .le. tlower .or. ciTemp .gt. tupper) weights%ciTemp = 0.d0
         if (oiTemp .le. tlower .or. oiTemp .gt. tupper) weights%oiTemp = 0.d0
 
-        if ((weights%oiiTemp + weights%siiTemp + weights%niiTemp + weights%ciTemp + weights%oiTemp) .gt. 0 .and. diagnostic_array(4) .eq. 0) then
+        if ((weights%oiiTemp + weights%siiTemp + weights%niiTemp + weights%ciTemp + weights%oiTemp) .gt. 0 .and. diagnostics%lowtemp .eq. 0) then
           lowtemp = (weights%oiiTemp*oiiTemp + weights%siiTemp*siiTemp + weights%niiTemp*niiTemp + weights%ciTemp*ciTemp + weights%oiTemp*oiTemp) / (weights%oiiTemp + weights%siiTemp + weights%niiTemp + weights%ciTemp + weights%oiTemp)
-        elseif (diagnostic_array(4) .gt. 0.0) then
-          lowtemp = diagnostic_array(4)
+        elseif (diagnostics%lowtemp .gt. 0.0) then
+          lowtemp = diagnostics%lowtemp
         else
           lowtemp = 10000.d0
         endif
@@ -371,10 +374,10 @@ use mod_hydrogen
         if (siiiIRDens .le. 0.d0) weights%siiiIRDens = 0.d0
         if (neiiiIRDens .le. 0.d0) weights%neiiiIRDens = 0.d0
 
-        if ((weights%cliiiDens + weights%ciiiDens + weights%arivDens) .eq. 0.d0 .and. diagnostic_array(2) .eq. 0) then
+        if ((weights%cliiiDens + weights%ciiiDens + weights%arivDens) .eq. 0.d0 .and. diagnostics%meddens .eq. 0) then
           meddens = lowdens
-        elseif (diagnostic_array(2) .gt. 0.0) then
-          meddens = diagnostic_array(2)
+        elseif (diagnostics%meddens .gt. 0.0) then
+          meddens = diagnostics%meddens
         else
           meddens = (weights%ciiiDens*ciiiDens + weights%cliiiDens*cliiiDens + weights%arivDens*arivDens + weights%oiiiIRDens*oiiiIRDens + weights%ariiiIRDens*ariiiIRDens + weights%siiiIRDens*siiiIRDens + weights%neiiiIRDens*neiiiIRDens) / (weights%cliiiDens + weights%ciiiDens + weights%arivDens + weights%oiiiIRDens + weights%ariiiIRDens + weights%siiiIRDens + weights%neiiiIRDens)
         endif
@@ -398,10 +401,10 @@ use mod_hydrogen
         if (ariiiTemp .le. tlower .or. ariiiTemp .gt. tupper) weights%ariiiTemp = 0.d0
         if (neiiiTemp .le. tlower .or. neiiiTemp .gt. tupper) weights%neiiiTemp = 0.d0
 
-        if ((weights%oiiitemp + weights%siiitemp + weights%ariiitemp + weights%neiiitemp) .gt. 0 .and. diagnostic_array(5) .eq. 0.0) then
+        if ((weights%oiiitemp + weights%siiitemp + weights%ariiitemp + weights%neiiitemp) .gt. 0 .and. diagnostics%medtemp .eq. 0.0) then
           medtemp = (weights%oiiitemp*oiiitemp + weights%siiitemp*siiitemp + weights%ariiitemp*ariiitemp + weights%neiiitemp*neiiitemp) / (weights%oiiitemp + weights%siiitemp + weights%ariiitemp + weights%neiiitemp)
-        elseif (diagnostic_array(5) .gt. 0.0) then
-          medtemp = diagnostic_array(5)
+        elseif (diagnostics%medtemp .gt. 0.0) then
+          medtemp = diagnostics%medtemp
         else
           medtemp = lowtemp
         endif
@@ -412,7 +415,7 @@ use mod_hydrogen
 
         !update extinction. DS 22/10/11
           meanextinction=0
-          call calc_extinction_coeffs(linelist,H_Balmer, c1, c2, c3, meanextinction, medtemp, lowdens,weights%ha,weights%hg,weights%hd)
+          call calc_extinction_coeffs(linelist,H_Balmer, c1, c2, c3, meanextinction, medtemp, lowdens,weights%ha,weights%hg,weights%hd,hidata)
 
           if (meanextinction .lt. 0.0 .or. isnan(meanextinction)) then
              meanextinction = 0.d0
@@ -442,8 +445,8 @@ use mod_hydrogen
 
         call get_diagnostic("[Ne IV]   ","1,2/                ","1,3/                ",neivNratio,"D",hightemp, neivDens,maxlevs,maxtemps,atomicdata,iion,tlower,tupper)
 
-        if (diagnostic_array(3) .gt. 0.0) then
-          highdens = diagnostic_array(3)
+        if (diagnostics%highdens .gt. 0.0) then
+          highdens = diagnostics%highdens
         elseif (neivdens .le. 0d0) then
           neivdens = 0d0
           highdens = meddens
@@ -461,10 +464,10 @@ use mod_hydrogen
         if (arvTemp .le. tlower .or. arvTemp .gt. tupper) weights%arvTemp = 0.d0
         if (nevTemp .le. tlower .or. nevTemp .gt. tupper) weights%nevTemp = 0.d0
 
-        if ((weights%arvTemp+weights%nevTemp) .gt. 0 .and. diagnostic_array(6) .eq. 0) then
+        if ((weights%arvTemp+weights%nevTemp) .gt. 0 .and. diagnostics%hightemp .eq. 0) then
           hightemp = (arvtemp*weights%arvTemp + nevtemp*weights%nevTemp) / (weights%arvTemp+weights%nevTemp)
-        elseif (diagnostic_array(6) .gt. 0.0) then
-          hightemp = diagnostic_array(6)
+        elseif (diagnostics%hightemp .gt. 0.0) then
+          hightemp = diagnostics%hightemp
         else
           hightemp = medtemp
         endif
@@ -673,13 +676,19 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
 ! Te(5876/4471) = a*r**4 + b*r**3 + c*r**2 + d*r + e + f/(r-g)
 ! a=6858; b=-99452.0; c=541944.8; d=-1317918.7; e=1210016.1; f=23.21; g=2.833
 ! accurate to +-50K
+
 ! Te(6678/4471) = a*r**2+b*r+c (r<0.775)
 !                 a*r**4+b**r*3+c**r*2+d**r+e (r>0.775)
 
+! Te(7281/6678) = a*r**4+b**r*3+c**r*2+d**r+e
+! coefficients are 2.695e7, -1.112e7, 1.824e6, 1.366e4, -1.025e3
+
         ratio_5876_4471=0.d0
         ratio_6678_4471=0.d0
+        ratio_7281_6678=0.d0
         Te_5876_4471=0.d0
         Te_6678_4471=0.d0
+        Te_7281_6678=0.d0
 
         if (HeI_lines(10) .gt. 0 .and. HeI_lines(15) .gt.0) then
           ratio_5876_4471=linelist(HeI_lines(15))%int_dered/linelist(HeI_lines(10))%int_dered
@@ -697,10 +706,17 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
           endif
         endif
 
+        if (HeI_lines(16).gt. 0 .and. HeI_lines(18).gt.0) then
+          ratio_7281_6678=linelist(HeI_lines(18))%int_dered/linelist(HeI_lines(16))%int_dered
+          Te_7281_6678 = 2.695e7*ratio_7281_6678**4 - 1.112e7*ratio_7281_6678**3 + 1.824e6*ratio_7281_6678**2 + 1.366e4*ratio_7281_6678 - 1.025e3
+        endif
+
         iteration_result%ratio_5876_4471=ratio_5876_4471
         iteration_result%ratio_6678_4471=ratio_6678_4471
+        iteration_result%ratio_7281_6678=ratio_7281_6678
         iteration_result%Te_5876_4471=Te_5876_4471
         iteration_result%Te_6678_4471=Te_6678_4471
+        iteration_result%Te_7281_6678=Te_7281_6678
 
 ! get te and ne from OII RL diagnostics if we have them
 
@@ -709,9 +725,9 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
         oii4662=0.d0
 
         do i=1,listlength
-          if (abs(linelist(i)%wavelength-4089.29) .lt. 0.005) oii4089=linelist(i)%int_dered
-          if (abs(linelist(i)%wavelength-4649.13) .lt. 0.005) oii4649=linelist(i)%int_dered
-          if (abs(linelist(i)%wavelength-4661.63) .lt. 0.005) oii4662=linelist(i)%int_dered
+          if (abs(linelist(i)%wavelength-4089.29) .lt. 0.006) oii4089=linelist(i)%int_dered
+          if (abs(linelist(i)%wavelength-4649.13) .lt. 0.006) oii4649=linelist(i)%int_dered
+          if (abs(linelist(i)%wavelength-4661.63) .lt. 0.006) oii4662=linelist(i)%int_dered
         enddo
 
         if (oii4089.gt.0 .and. oii4649.gt.0 .and. oii4662.gt.0) then
@@ -729,7 +745,6 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
 
         do i = 1,size(ILs)
           if (ILs(i)%location .gt. 0) then !the line is in the spectrum
-             linelist(ILs(i)%location)%name=ILs(i)%ion
              if (ILs(i)%zone .eq. "low ") then
                call get_abundance(ILs(i)%ion, ILs(i)%transition, lowtemp, lowdens,linelist(ILs(i)%location)%int_dered, linelist(ILs(i)%location)%abundance,maxlevs,maxtemps,atomicdata,iion)
              elseif (ILs(i)%zone .eq. "med ") then
@@ -791,87 +806,75 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
         NOabundCEL = oiCELabund
 
 ! now get abundances for ORLs
-! o2+
-       call oii_rec_lines(medtemp,meddens,1.d0,oiiRLs)
 
+       call get_aeff_hb(medtemp,meddens,aeff_hb,em_hb)
+
+! o2+
+!       call oii_rec_lines(medtemp,meddens,1.d0,oiiRLs,aeff_hb,em_hb)
+       call oii_rec_lines_s2017(medtemp,meddens,1.d0,oiiRLs_s2017,em_hb)
        do i = 1,listlength
-         do j = 1,size(oiiRLs)
-          if (abs(linelist(i)%wavelength-oiiRLs(j)%Wave) .le. 0.005) then
-            oiiRLs(j)%Obs = linelist(i)%int_dered
-            if (oiiRLs(j)%Int .eq. 0.0) then
-              oiiRLs(j)%abundance = 0.D0
+         do j = 1,size(oiiRLs_s2017)
+          if (abs(linelist(i)%wavelength-oiiRLs_s2017(j)%Wave) .le. 0.006) then
+            oiiRLs_s2017(j)%Obs = linelist(i)%int_dered
+            if (oiiRLs_s2017(j)%Int .eq. 0.0) then
+              oiiRLs_s2017(j)%abundance = 0.D0
             else
-              oiiRLs(j)%abundance = oiiRLs(j)%obs/oiiRLs(j)%Int
+              oiiRLs_s2017(j)%abundance = oiiRLs_s2017(j)%obs/oiiRLs_s2017(j)%Int
             endif
-            linelist(i)%abundance = oiiRLs(j)%abundance
-            linelist(i)%name="O II       "
-            linelist(i)%latextext="O~{\sc ii}     "
+            linelist(i)%abundance = oiiRLs_s2017(j)%abundance
           endif
          enddo
        enddo
 
 !N2+
 
-       call nii_rec_lines(medtemp,meddens,1.d0,niiRLs)
+       call nii_rec_lines(medtemp,1.d0,niiRLs,em_hb)
 
        do i = 1,listlength
          do j = 1,size(niiRLs)
-          if (abs(linelist(i)%wavelength-niiRLs(j)%Wave) .le. 0.005) then
+          if (abs(linelist(i)%wavelength-niiRLs(j)%Wave) .le. 0.006) then
             niiRLs(j)%Obs = linelist(i)%int_dered
             niiRLs(j)%abundance = niiRLs(j)%obs/niiRLs(j)%Int
             linelist(i)%abundance = niiRLs(j)%abundance
-            linelist(i)%name="N II       "
-            linelist(i)%latextext="N~{\sc ii}     "
           endif
          enddo
        enddo
 
 !C2+
-       call cii_rec_lines(medtemp,meddens,1.d0,ciiRLs)
+       call cii_rec_lines(medtemp,1.d0,ciiRLs,aeff_hb)
 
        do i = 1,listlength
          do j = 1,size(ciiRLs)
-          if (abs(linelist(i)%wavelength-ciiRLs(j)%Wave) .le. 0.005) then
+          if (abs(linelist(i)%wavelength-ciiRLs(j)%Wave) .le. 0.006) then
             ciiRLs(j)%Obs = linelist(i)%int_dered
             ciiRLs(j)%abundance = ciiRLs(j)%obs/ciiRLs(j)%Int
             linelist(i)%abundance = ciiRLs(j)%abundance
-            linelist(i)%name="C II       "
-            linelist(i)%latextext="C~{\sc ii}     "
           endif
          enddo
        enddo
 
 !Ne2+
-       call neii_rec_lines(medtemp,meddens,1.d0,neiiRLs)
+       call neii_rec_lines(medtemp,1.d0,neiiRLs,aeff_hb)
 
        do i = 1,listlength
          do j = 1,size(neiiRLs)
-          if (abs(linelist(i)%wavelength-neiiRLs(j)%Wave) .le. 0.005) then
+          if (abs(linelist(i)%wavelength-neiiRLs(j)%Wave) .le. 0.006) then
             neiiRLs(j)%Obs = linelist(i)%int_dered
             neiiRLs(j)%abundance = neiiRLs(j)%obs/neiiRLs(j)%Int
             linelist(i)%abundance = neiiRLs(j)%abundance
-            linelist(i)%name="Ne II      "
-            linelist(i)%latextext="Ne~{\sc ii}    "
           endif
          enddo
        enddo
 
 !C3+, N3+
-       call xiii_rec_lines(medtemp,meddens,1.d0,xiiiRLs)
+       call xiii_rec_lines(medtemp,1.d0,xiiiRLs,aeff_hb)
 
        do i = 1,listlength
          do j = 1,size(xiiiRLs)
-          if (abs(linelist(i)%wavelength-xiiiRLs(j)%Wave) .le. 0.005) then
+          if (abs(linelist(i)%wavelength-xiiiRLs(j)%Wave) .le. 0.006) then
             xiiiRLs(j)%Obs = linelist(i)%int_dered
             xiiiRLs(j)%abundance = xiiiRLs(j)%obs/xiiiRLs(j)%Int
             linelist(i)%abundance = xiiiRLs(j)%abundance
-            if (j .le. 4) then
-              linelist(i)%name="C III      "
-              linelist(i)%latextext="C~{\sc iii}    "
-            else
-              linelist(i)%name="N III      "
-              linelist(i)%latextext="N~{\sc iii}    "
-            endif
           endif
          enddo
        enddo
@@ -933,26 +936,38 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
         niiRLabund = 0.D0
       endif
 
+! simple checks for the reliability of the abundance: flagged as unreliable if any of the following apply:
+!  - only one multiplet detected
+!  - any multiplets have >3x the abundance of the lowest value
+! todo: report which multiplets exceed 3x
+
+      if (maxval(niimultiplets%abundance,mask=(niimultiplets%weight.gt.0.d0)) .gt. 3.0*minval(niimultiplets%abundance,mask=(niimultiplets%weight.gt.0.d0 .and. niimultiplets%abundance.gt.0.d0)) .or. count(niimultiplets%abundance .gt. 0.d0) .le. 1) then
+        niiRLreliable = .false.
+      else
+        niiRLreliable = .true.
+      endif
+
+
   endif
 
 !oii recombination lines
 
-  if (sum(oiiRLs%Int).gt. 0) then
+  if (sum(oiiRLs_s2017%Int).gt. 0) then
 
-      oiimultiplets%Multiplet = (/" V1    "," V2    "," V5    " ," V10   "," V11   "," V12   "," V19   "," V20   "," V25   "," V28   "," V33   "," 3d-4f "/)
+      oiimultiplets%Multiplet = (/"V1     ","V2     ","V5     " ,"V10    ","V11    ","V12    ","V19    ","V20    ","V25    ","V28    ","V33    "," 3d-4f "/)
       oiimultiplets%weight = (/ weights%oiiV1, weights%oiiV2, weights%oiiV5, weights%oiiV10, weights%oiiV11, weights%oiiV12, weights%oiiV19, weights%oiiV20, weights%oiiV25, weights%oiiV28, weights%oiiV33, weights%oii3d4f /)
 
 ! get observed and predicted coadded intensity for each multiplet
 
       do j = 1,11
-        oiimultiplets(j)%coadded_observed=sum(oiiRLs%obs,mask=oiiRLs%Mult .eq. oiimultiplets(j)%Multiplet .and. oiiRLs%obs .gt. 0)
-        oiimultiplets(j)%coadded_predicted=sum(oiiRLs%Int,mask=oiiRLs%Mult .eq. oiimultiplets(j)%Multiplet .and. oiiRLs%obs .gt. 0)
+        oiimultiplets(j)%coadded_observed=sum(oiiRLs_s2017%obs,mask=oiiRLs_s2017%Mult .eq. oiimultiplets(j)%Multiplet .and. oiiRLs_s2017%obs .gt. 0)
+        oiimultiplets(j)%coadded_predicted=sum(oiiRLs_s2017%Int,mask=oiiRLs_s2017%Mult .eq. oiimultiplets(j)%Multiplet .and. oiiRLs_s2017%obs .gt. 0)
       enddo
 
 ! 3d-4f transitions
 
-      oiimultiplets(12)%coadded_observed=sum(oiiRLs%obs,mask=oiiRLs%Term1(4:5) .eq. "3d" .and. oiiRLs%Term2(3:4) .eq. "4f" .and. oiiRLs%Mult .ne. "       " .and. oiiRLs%obs .gt. 0)
-      oiimultiplets(12)%coadded_predicted=sum(oiiRLs%Int,mask=oiiRLs%Term1(4:5) .eq. "3d" .and. oiiRLs%Term2(3:4) .eq. "4f" .and. oiiRLs%Mult .ne. "       " .and. oiiRLs%obs .gt. 0)
+      oiimultiplets(12)%coadded_observed=sum(oiiRLs_s2017%obs,mask=oiiRLs_s2017%Term1(1:2) .eq. "3d" .and. oiiRLs_s2017%Term2(1:2) .eq. "4f" .and. oiiRLs_s2017%Mult .ne. "       " .and. oiiRLs_s2017%obs .gt. 0)
+      oiimultiplets(12)%coadded_predicted=sum(oiiRLs_s2017%Int,mask=oiiRLs_s2017%Term1(1:2) .eq. "3d" .and. oiiRLs_s2017%Term2(1:2) .eq. "4f" .and. oiiRLs_s2017%Mult .ne. "       " .and. oiiRLs_s2017%obs .gt. 0)
 
 !get multiplet abundances from coadded intensity
 
@@ -979,6 +994,18 @@ iteration_result(1)%NeV_temp_ratio = nevTratio
         oiiRLabund = 0.D0
       endif
 
+! simple checks for the reliability of the abundance: flagged as unreliable if any of the following apply:
+!  - only one multiplet detected
+!  - any multiplets have >3x the abundance of the lowest value
+!  - neither V1 nor V10 is detected
+! todo: report which multiplets exceed 3x
+
+      if (maxval(oiimultiplets%abundance,mask=(oiimultiplets%weight.gt.0.d0)) .gt. 3.0*minval(oiimultiplets%abundance,mask=(oiimultiplets%weight.gt.0.d0 .and. oiimultiplets%abundance.gt.0.d0)) .or. (oiimultiplets(1)%abundance.eq.0.d0 .and. oiimultiplets(4)%abundance.eq.0.d0) .or. count(oiimultiplets%abundance .gt. 0.d0) .le. 1) then
+        oiiRLreliable = .false.
+      else
+        oiiRLreliable = .true.
+      endif
+
    endif
 
 !neii recombination lines
@@ -1242,7 +1269,7 @@ elseif (switch_icf .eq. "P") then
        Hetotabund = RLicfHe * Heiabund ! (21) - note that this icf is not meaningful if only S+ is seen
      endif
 
-elseif (switch_icf .eq. "D") then
+elseif (switch_icf .eq. "D" .or. switch_icf .eq. "E") then
 !Delgado-Inglada 2014
 
 !starting definitions
@@ -1275,17 +1302,31 @@ elseif (switch_icf .eq. "D") then
   endif
 
 !nitrogen
-!equation 14:
-
-  if (niiCELabund .gt. 0.D0 .and. oiiCELabund .gt. 0.D0 .and. upsilon .gt. 0.D0) then
-    CELicfN = 10.**(-0.16*oICFfactor*(1+log10(upsilon)))
-    nabundCEL = CELicfN * niiCELabund * OabundCEL / oiiCELabund
-  elseif (niiCELabund .gt. 0.D0 .and. oiiCELabund .gt. 0.D0 .and. upsilon .eq. 0.D0) then
-    CELicfN = 10**(0.64*Oicffactor)
-    nabundCEL = CELicfN * niiCELabund * OabundCEL / oiiCELabund
+!use the classical icf if DI14mod selected
+  if (switch_icf=="E") then
+    NabundCEL = 0.d0
+    if (niiCELabund .ge. 1e-20 .and. niiiUVCELabund .ge. 1e-20 .and. nivCELabund .ge. 1e-20) then !all ionisation stages seen
+      CELicfN = 1.
+      NabundCEL = niiCELabund + niiiCELabund + nivCELabund
+    elseif (niiCELabund .ge. 1e-20 .and. niiiUVCELabund .lt. 1e-20 .and. nivCELabund .ge. 1e-20) then !no N2+ seen
+      CELicfN = 1.5
+      NabundCEL = 1.5*(niiCELabund + nivCELabund)
+    elseif (niiCELabund .ge. 1e-20 .and. niiiUVCELabund .lt. 1e-20 .and. nivCELabund .lt. 1e-20) then !Only N+ seen
+      CELicfN = OabundCEL/oiiCELabund
+      NabundCEL = niiCELabund * CELicfN
+    endif
   else
-    CELicfN = 1.0
-    nabundCEL = 0.D0
+!use DI14 as published: equation 14:
+    if (niiCELabund .gt. 0.D0 .and. oiiCELabund .gt. 0.D0 .and. upsilon .gt. 0.D0) then
+      CELicfN = 10.**(-0.16*oICFfactor*(1+log10(upsilon)))
+      nabundCEL = CELicfN * niiCELabund * OabundCEL / oiiCELabund
+    elseif (niiCELabund .gt. 0.D0 .and. oiiCELabund .gt. 0.D0 .and. upsilon .eq. 0.D0) then
+      CELicfN = 10**(0.64*Oicffactor)
+      nabundCEL = CELicfN * niiCELabund * OabundCEL / oiiCELabund
+    else
+      CELicfN = 1.0
+      nabundCEL = 0.D0
+    endif
   endif
 
 !neon
@@ -1381,9 +1422,16 @@ endif
      endif
 
 !Nitrogen
+!from CELs we have N+ and N+/N, from RLs we have N2+ + N3+
+!so assuming no N4+, RLicf(N) = CELicf(N)/(CELicf(N)-1)
+!if CELicfN = 1.0, then there should be no N2+ or N3+
 
-     RLicfN = 1.0
-     NabundRL = niiRLabund + niiiRLabund
+     if (CELicfN .gt. 1.0) then
+       RLicfN = NabundCEL/(NabundCEL-niiCELabund)
+     else
+       RLicfN = 1.0
+     endif
+     NabundRL = RLicfN*(niiRLabund + niiiRLabund)
 
 !Carbon
 
@@ -1564,6 +1612,8 @@ endif
         iteration_result(1)%nii_v20_abund_orl = niimultiplets(5)%abundance
         iteration_result(1)%nii_v28_abund_orl = niimultiplets(6)%abundance
         iteration_result(1)%nii_3d4f_abund_orl = niimultiplets(7)%abundance
+        iteration_result(1)%oiiRLreliable = oiiRLreliable
+        iteration_result(1)%niiRLreliable = niiRLreliable
 
 !Strong line methods
 
@@ -1627,13 +1677,13 @@ endif
 
 !abundance discrepancy factors
 
-  if (oiiiCELabund .gt. 0) then
+  if (oiiiCELabund .gt. 0 .and. oiiRLreliable) then
     adfO2plus = oiiRLabund/oiiiCELabund
   else
     adfO2plus = 0.d0
   endif
 
-  if (oabundCEL .gt. 0) then
+  if (oabundCEL .gt. 0 .and. oiiRLreliable) then
     adfO = OabundRL/OabundCEL
   else
     adfO = 0.d0
@@ -1653,13 +1703,13 @@ endif
   endif
 
 
-  if (NiiiCELabund .gt. 0) then
+  if (NiiiCELabund .gt. 0 .and. niiRLreliable) then
     adfN2plus = NiiRLabund/NiiiCELabund
   else
     adfN2plus = 0.d0
   endif
 
-  if (NabundCEL .gt. 0) then
+  if (NabundCEL .gt. 0 .and. niiRLreliable) then
     adfN = NabundRL/NabundCEL
   else
     adfN = 0.d0
@@ -1782,8 +1832,6 @@ contains
 
 subroutine get_diag(name1, name2, diag)
         implicit none
-        integer, parameter :: dp = kind(1.d0)
-
         character(len=11) :: name1, name2
         real(kind=dp) :: flux_no1, flux_no2
         real(kind=dp) :: diag
@@ -1807,8 +1855,6 @@ end subroutine get_diag
 subroutine get_Tdiag(name1, name2, name3, ion, ratio)
         !this routine gets the ratio for nebular to auroral diagnostics.  In case one of nebular pair is not observed, it assumes the intensity of the other is given by the theoretical ratio
         implicit none
-        integer, parameter :: dp = kind(1.d0)
-
         character(len=11) :: name1, name2, name3
         character(len=20) :: ion
         integer :: ion_no1, ion_no2, ion_no3
@@ -1909,7 +1955,7 @@ subroutine get_average_abundance(startion,endion,abundance)
 end subroutine get_average_abundance
 
 subroutine fix_siii
-!SIII 9069 and 9531 doublet is a special case due to absorption lines. See Liu, Barlow etc 1995 (Far Red IR Lines in a PN), one of 9069/9531 is ALWAYS absorbed but the other is then always fine.  We can tell which is is by looking at the ratio 9069/9531.
+!SIII 9069 and 9531 doublet is a special case due to absorption lines. See Liu et al 1995 (https://ui.adsabs.harvard.edu/abs/1995MNRAS.273...47L): one of 9069/9531 is ALWAYS absorbed but the other is then always fine.  We can tell which is is by looking at the ratio 9069/9531.
 
         implicit none
         real(kind=dp) :: ratio!,factor
