@@ -1,33 +1,28 @@
 !filereading.f90, routines to read in the input file and various atomic data files, and also lookup routines for line identification
 !(C) Roger Wesson, Dave Stock, Peter Scicluna
 module mod_abundIO
-use mod_abundtypes
+use mod_types
 use mod_atomicdata
+use mod_globals
+use mod_functions
+
 implicit none
-private :: dp
-integer, parameter :: dp = kind(1.d0)
 
 contains
 
-subroutine read_linelist(filename,linelist,listlength,ncols,errstat)
+subroutine read_text_linelist(linelist,listlength,ncols,runs)
         implicit none
-        integer :: i, j, io, nlines, listlength, errstat, ncols
+        integer :: i, j, io, nlines, listlength, errstat, ncols, runs
         character(len=1) :: blank
         type(line), dimension(:), allocatable :: linelist
-        character(len=512) :: filename, rowdata
+        character(len=512) :: rowdata
         character(len=15),dimension(6) :: invar !line fluxes and uncertainties are read as strings into these variables
         real(kind=dp),dimension(7) :: rowdata2 !to check number of columns, first row of file is read as character, then read as real into this array
-
-        type neat_line
-          real(kind=dp) :: wavelength
-          character(len=85) :: linedata
-        end type neat_line
-
         type(neat_line), dimension(:), allocatable :: neatlines
 
 !debugging
 #ifdef CO
-        print *,"subroutine: read_linelist"
+        print *,"subroutine: read_text_linelist"
 #endif
 
 ! first get number of rows
@@ -58,12 +53,14 @@ subroutine read_linelist(filename,linelist,listlength,ncols,errstat)
         linelist%blend_intensity=0.d0
         linelist%blend_int_err=0d0
         linelist%zone='    '
-        linelist%name='           '
         linelist%transition='                    '
         linelist%location=0
         linelist%ion='          '
-        linelist%latextext='               '
-        linelist%linedata='                                                                           '
+        linelist%multiplet='            '
+        linelist%lowerterm='            '
+        linelist%upperterm='            '
+        linelist%g1=0
+        linelist%g2=0
 
 ! now count the columns
 ! if 2 - assume rest wavelength and intensity, read in and restrict to single iteration
@@ -75,7 +72,7 @@ subroutine read_linelist(filename,linelist,listlength,ncols,errstat)
 
         do while (IO .ge. 0)
           read (199,"(A)") rowdata
-          if (index(rowdata,"#") .ne. 1) then
+          if (index(adjustl(rowdata),"#") .ne. 1) then
             read (rowdata,*,end=113) rowdata2(:)
           endif
         enddo
@@ -88,7 +85,7 @@ subroutine read_linelist(filename,linelist,listlength,ncols,errstat)
         DO while (I.le.listlength)
           read(199,"(A)",end=110) rowdata
 
-          if (index(rowdata,"#") .ne. 1) then !not a comment, read in the columns
+          if (index(adjustl(rowdata),"#") .ne. 1) then !not a comment, read in the columns
             read(rowdata,*) invar(1:ncols)
           else !do nothing with comment lines
             cycle
@@ -117,7 +114,6 @@ subroutine read_linelist(filename,linelist,listlength,ncols,errstat)
               read (invar(4),*) linelist(i)%int_err
             endif
           endif
-          linelist(i)%latextext = ""
           i=i+1
         enddo
 
@@ -152,35 +148,199 @@ subroutine read_linelist(filename,linelist,listlength,ncols,errstat)
         101 nlines=I-1
 
 !then allocate and read
+!todo: read wavelength only, then if matched, rewind and read rest
         allocate (neatlines(nlines))
 
         rewind (100)
         DO I=1,nlines
-          read(100,"(F8.2,A85)",end=102) neatlines(i)%wavelength,neatlines(i)%linedata
+          read(100,"(F8.2,X,A12,X,A12,X,A12,X,A12,X,I12,X,I9)",end=102) neatlines(i)%wavelength,neatlines(i)%ion,neatlines(i)%multiplet,neatlines(i)%lowerterm,neatlines(i)%upperterm,neatlines(i)%g1,neatlines(i)%g2
           do j=1,listlength
             if (abs(linelist(j)%wavelength - neatlines(i)%wavelength) .lt. 0.011) then
-              linelist(j)%linedata = neatlines(i)%linedata
-              linelist(j)%name = latextoplain(linelist(j)%linedata(3:19))
+              linelist(j)%ion = neatlines(i)%ion
+              linelist(j)%multiplet = neatlines(i)%multiplet
+              linelist(j)%lowerterm = neatlines(i)%lowerterm
+              linelist(j)%upperterm = neatlines(i)%upperterm
+              linelist(j)%g1 = neatlines(i)%g1
+              linelist(j)%g2 = neatlines(i)%g2
             endif
           enddo
         enddo
         102 print *
         close(100)
 
+!check for any duplicate lines, warn if found
+
+        do i=1,listlength
+          do j=1,listlength
+            if (linelist(i)%wavelength .eq. linelist(j)%wavelength .and. i.ne.j) then
+              errstat=errstat+8
+              goto 155
+            endif
+          enddo
+        enddo
+
+155     continue
+
+!fill observed wavelength column if necessary
+
+        if (ncols.lt.4) then
+          linelist%wavelength_observed=linelist%wavelength
+        endif
+
 !fix some common blends.  If first member of blend is in the list but the second is not, or if second is present but with zero flux, then presume the feature is blended.
 
-        call fix_blend(3726.03d0,3728.82d0,3727.00d0,linelist)
-        call fix_blend(7318.92d0,7319.99d0,7319.45d0,linelist)
-        call fix_blend(7329.67d0,7330.73d0,7330.20d0,linelist)
-        call fix_blend(1906.68d0,1908.73d0,1909.00d0,linelist)
-        call fix_blend(2422.36d0,2425.01d0,2424.00d0,linelist)
-        call fix_blend(4714.17d0,4715.66d0,4715.21d0,linelist)
-        call fix_blend(4724.15d0,4725.62d0,4724.89d0,linelist)
-        call fix_blend(1483.32d0,1486.50d0,1485.00d0,linelist)
+        call fix_blends(linelist)
+
+! error status was incremented by 1, 2 or 4 depending on error so btest function determines which errors were encountered
+! test the two fatal errors first, then the warning.
+
+        if (btest(errstat,0)) then
+                print *,gettime(),"error: line list reading failed"
+                print *,"            This can happen if it doesn't have three columns"
+                call exit(105)
+        elseif (btest(errstat,1)) then
+                print*, gettime(),"cheese shop error - no inputs"
+                call exit(100)
+        elseif (btest(errstat,2) .and. runs .gt. 1) then !only relevant if uncertainties were requested
+                print*, gettime(),"warning: no uncertainties given, arbitrarily assuming 10 per cent for everything"
+        elseif (btest(errstat,3)) then
+                print *,gettime(),"warning: duplicate line measurements detected. Only the second measurement is used in abundance determinations"
+        else
+                print "(X,A,A,A,A,I3,A)", gettime(),"line list file ",trim(filename)," read successfully (",listlength," lines)"
+        endif
 
 !todo:remove elements from the array?
 
-end subroutine read_linelist
+end subroutine read_text_linelist
+
+subroutine read_fits_linelist(linelist,listlength,ncols)
+! read a FITS file generated by ALFA or a previous NEAT analysis
+! extension LINES contains rest wavelength, observed wavelength, flux, uncertainty, (peak, FWHM if generated by ALFA), transition data
+  implicit none
+  type(line), dimension(:), allocatable :: linelist
+  logical, dimension(:), allocatable :: blends
+  integer :: listlength, ncols, i
+
+! cfitsio variables
+
+  integer :: status,unit,readwrite,blocksize
+  character(len=30) :: cfitsioerror
+  logical :: anyf
+
+  status=0
+  readwrite=0
+  ncols=12
+
+! open the file
+  call ftgiou(unit,status)
+  call ftopen(unit,filename,readwrite,blocksize,status)
+
+! check if already analysed
+  call ftmnhd(unit,-1,"RESULTS",0,status)
+  if (status.eq.0) then
+    print *,gettime(),"line list already analysed - previous results will be overwritten"
+  endif
+  status=0
+
+! go to the LINES extension
+  call ftmnhd(unit,-1,"LINES",0,status)
+  if (status.ne.0) then
+    call ftgerr(status,cfitsioerror)
+    print *,gettime(),"error opening file ",trim(filename),": ",status,cfitsioerror
+    call exit(106)
+  endif
+
+! get number of rows, allocate, initialise
+
+  call ftgnrw(unit,listlength,status)
+
+  allocate(linelist(listlength))
+  allocate(blends(listlength))
+
+  linelist%intensity = 0.D0
+  linelist%abundance = 0.D0
+  linelist%weight = 0.d0
+  linelist%freq=0d0
+  linelist%wavelength=0d0
+  linelist%wavelength_observed=0d0
+  linelist%int_dered=0d0
+  linelist%int_err=0d0
+  linelist%blend_intensity=0.d0
+  linelist%blend_int_err=0d0
+  linelist%zone='    '
+  linelist%transition='                    '
+  linelist%location=0
+  linelist%ion='          '
+  linelist%multiplet='            '
+  linelist%lowerterm='            '
+  linelist%upperterm='            '
+  linelist%g1=0
+  linelist%g2=0
+
+! read columns. todo: need ftgcfd?
+! cols from ALFA are obswlen, restwlen, int, int_err, peak, fwhm, ion, multiplet, lowerterm, upperterm, g1, g2
+! 5 & 6 are peak and FWHM
+
+  call ftgcvd(unit,1,1,1,listlength,0,linelist%wavelength_observed,anyf,status)
+  call ftgcvd(unit,2,1,1,listlength,0,linelist%wavelength,anyf,status)
+  call ftgcvd(unit,3,1,1,listlength,0,linelist%intensity,anyf,status)
+  call ftgcvd(unit,4,1,1,listlength,0,linelist%int_err,anyf,status)
+  call ftgcvs(unit,7,1,1,listlength,"",linelist%ion,anyf,status)
+  call ftgcvs(unit,8,1,1,listlength,"",linelist%multiplet,anyf,status)
+  call ftgcvs(unit,9,1,1,listlength,"",linelist%lowerterm,anyf,status)
+  call ftgcvs(unit,10,1,1,listlength,"",linelist%upperterm,anyf,status)
+  call ftgcvj(unit,11,1,1,listlength,0,linelist%g1,anyf,status)
+  call ftgcvj(unit,12,1,1,listlength,0,linelist%g2,anyf,status)
+
+! break if there were errors
+
+  if (status.ne.0) then
+    call ftgerr(status,cfitsioerror)
+    print *,gettime(),"error opening file ",trim(filename),": ",status,cfitsioerror
+    call exit(107)
+  endif
+
+! close
+
+  call ftclos(unit, status)
+  call ftfiou(unit, status)
+
+! blends
+
+  do i=1,size(linelist)
+    if (linelist(i)%wavelength_observed.eq.0.d0) then
+      linelist(i-1)%blend_intensity=linelist(i-1)%intensity
+      linelist(i-1)%blend_int_err=linelist(i-1)%int_err
+      linelist(i-1:i)%intensity = 0.d0
+      linelist(i-1:i)%int_err = 0.d0
+    endif
+  enddo
+
+  call fix_blends(linelist)
+
+! ALFA writes out single precision real numbers (double precision makes it 30% slower)
+! but reading in double precision from FITS file results in rounding errors
+! so, this obscenity seems like the best way to get wavelengths to 2dp, to ensure that all lines are identified correctly
+
+  linelist%wavelength=0.01d0*(nint(100.d0*linelist%wavelength))
+
+end subroutine read_fits_linelist
+
+subroutine fix_blends(linelist)
+
+  implicit none
+  type(line),dimension(:) :: linelist
+
+  call fix_blend(3726.03d0,3728.82d0,3727.00d0,linelist)
+  call fix_blend(7318.92d0,7319.99d0,7319.45d0,linelist)
+  call fix_blend(7329.67d0,7330.73d0,7330.20d0,linelist)
+  call fix_blend(1906.68d0,1908.73d0,1909.00d0,linelist)
+  call fix_blend(2422.36d0,2425.01d0,2424.00d0,linelist)
+  call fix_blend(4714.17d0,4715.66d0,4715.21d0,linelist)
+  call fix_blend(4724.15d0,4725.62d0,4724.89d0,linelist)
+  call fix_blend(1483.32d0,1486.50d0,1485.00d0,linelist)
+
+end subroutine fix_blends
 
 subroutine read_celdata(ILs, ionlist)
 !this subroutine reads in the parameters of CEL diagnostic lines - the line id, ion name, wavelength, energy levels, and zone.  the index locating the line in the main line list is filled in later.
@@ -309,7 +469,7 @@ integer function get_ion(ionname, ILs)
 
         get_ion = 0
         print *,"           Nudge Nudge, wink, wink error. Ion not found, say no more. ", ionname,i
-        call exit(1)
+        call exit(202)
 
 end function
 
@@ -339,7 +499,7 @@ integer function get_atomicdata(ionname, atomicdatatable)
 end function
 
 subroutine fix_blend(line1, line2, blendwavelength, linelist)
-
+! rewrites blended CELs as a single line such that abundances can be straightforwardly calculated
         implicit none
         type(line), dimension(:) :: linelist
         real(kind=dp) :: line1, line2, blendwavelength,line2flux
@@ -359,6 +519,8 @@ subroutine fix_blend(line1, line2, blendwavelength, linelist)
 
         if ( (location1 .gt. 0 .and. location2 .eq. 0) .or. (location1 .gt. 0 .and. location2 .gt. 0 .and. line2flux .eq. 0.d0) ) then
           linelist(location1)%wavelength = blendwavelength
+          linelist(location1)%intensity = linelist(location1)%blend_intensity ! copy blend intensity back to intensity so it is used in abundance calculation
+          linelist(location1)%blend_intensity = 0.d0
           print "(12X,A,F7.2,A,F7.2)","unresolved blend: changed wavelength ",line1," to ",blendwavelength
         endif
 
@@ -416,7 +578,6 @@ end subroutine get_cels
 subroutine get_H(H_Balmer, H_paschen, linelist)
 !index the location of Balmer and Paschen lines in the main array
   implicit none
-  integer, parameter :: dp = kind(1.d0)
   integer, dimension(3:40), intent(out) :: H_balmer ! indexing starts at 3 so that it represents the upper level of the line
   integer, dimension(4:39), intent(out) :: H_paschen ! indexing starts at 4 for the same reason
   real(kind=dp), dimension(3:40) :: balmerwavelengths
@@ -440,8 +601,6 @@ subroutine get_H(H_Balmer, H_paschen, linelist)
     do j=3,40
       if (abs(linelist(i)%wavelength - balmerwavelengths(j)) .lt. 0.005) then
         H_balmer(j)=i
-        linelist(i)%name="H I"
-        linelist(i)%latextext = "H~{\sc i}"
         cycle
       endif
     enddo
@@ -451,8 +610,6 @@ subroutine get_H(H_Balmer, H_paschen, linelist)
     do j=4,39
       if (abs(linelist(i)%wavelength - paschenwavelengths(j)) .lt. 0.005) then
         H_paschen(j)=i
-        linelist(i)%name="H I"
-        linelist(i)%latextext = "H~{\sc i}"
         cycle
       endif
     enddo
@@ -463,7 +620,6 @@ end subroutine get_H
 subroutine get_HeI(HeI_lines, linelist)
 !index the locations of He I lines in the main linelist array
         implicit none
-        integer, parameter :: dp = kind(1.d0)
         integer, dimension(44), intent(out) :: HeI_lines
         type(line), dimension(:) :: linelist
         real(kind=dp), dimension(44) :: wavelengths
@@ -482,8 +638,6 @@ subroutine get_HeI(HeI_lines, linelist)
           do j = 1, size(linelist)
             if(abs(linelist(j)%wavelength - wavelengths(i)) .lt.  0.005) then
               Hei_lines(i) = j
-              linelist(j)%name="He I"
-              linelist(j)%latextext = "He~{\sc i}"
               cycle
             endif
           enddo
@@ -494,7 +648,6 @@ end subroutine get_HeI
 subroutine get_HeII(HeII_lines, linelist)
 !index the location of He II lines (only 4686 at the moment) in the linelist array
         implicit none
-        integer, parameter :: dp = kind(1.d0)
         integer, dimension(20,2:6), intent(out) :: HeII_lines ! array indices represent quantum numbers
         type(line), dimension(:) :: linelist
         real(kind=dp), dimension(20,2:6) :: wavelengths
@@ -569,8 +722,6 @@ subroutine get_HeII(HeII_lines, linelist)
             do k = 1, size(linelist)
               if(abs(linelist(k)%wavelength - wavelengths(j,i)) .lt.  0.005) then
                 Heii_lines(j,i) = k
-                linelist(k)%name="He II"
-                linelist(k)%latextext = "He~{\sc ii}"
                 cycle
               endif
             enddo
@@ -579,49 +730,11 @@ subroutine get_HeII(HeII_lines, linelist)
 
 end subroutine get_HeII
 
-character(len=11) function latextoplain(text)
-!converts latex-formatted ion name from linelist%linedata to plain text
-!eg [O~{\sc iii}] -> [O II]
-
-implicit none
-character(len=17) :: text,part1,part2,part3
-integer :: i
-
-if (index(text,"~{\sc") .gt. 0) then
-
-  !123456789012
-  ![O~\sc iii}]
-
-  part1="                 "
-  part2="                 "
-  part3="                 "
-
-  part1=text(1:index(text,"~{\sc")-1)
-  part2=text(index(text,"~{\sc")+5:index(text,"}")-1)
-  part3=text(index(text,"}")+1:)
-
-  do i=1,len(part2)
-    if (iachar(part2(i:i)).ge.iachar("a") .and. iachar(part2(i:i)).le.iachar("z")) then !convert to lower case
-      part2(i:i) = achar(iachar(part2(i:i))-32)
-    endif
-  enddo
-
-  latextoplain=trim(part1)//trim(part2)//trim(part3)
-
-else
-  latextoplain=text(1:11)
-endif
-
-end function
-
 end module
 
 module mod_atomic_read
 use mod_atomicdata
 
-private :: dp
-integer, parameter :: dp = kind(1.d0)
-
 contains
 subroutine read_atomic_data(ion)
 use mod_atomicdata
@@ -630,7 +743,7 @@ use mod_atomicdata
     integer :: I,J,K,L,N,NCOMS,ID(2),JD(2),KP1,NLEV1,GX,ionl,dummy
     character(len=1) :: comments(78)
     character(len=10) :: ionname
-    character(len=128) :: filename
+    character(len=128) :: atomicdatafile
     real(kind=dp) :: WN,AX,QX
 
 !debugging
@@ -645,23 +758,23 @@ use mod_atomicdata
     ionl = index(ionname,' ') - 1
 !get file name by removing [, " " and ] and converting to lower case
 
-    filename=""
+    atomicdatafile=""
     i=1
     j=1
 
     do while (i .le. len(ionname))
       if (iachar(ionname(i:i)).ge.iachar("A") .and. iachar(ionname(i:i)).le.iachar("Z")) then !convert to lower case
-        filename(j:j) = achar(iachar(ionname(i:i))+32)
+        atomicdatafile(j:j) = achar(iachar(ionname(i:i))+32)
         j=j+1
       elseif (ionname(i:i).ne."[".and.ionname(i:i).ne." ".and.ionname(i:i).ne."]") then !remove
-        filename(j:j)=ionname(i:i)
+        atomicdatafile(j:j)=ionname(i:i)
         j=j+1
       endif
         i=i+1
     enddo
 
-    filename = trim(PREFIX)//'/share/neat/'//trim(filename)//'.dat'
-    open(unit=1, status = 'old', file=filename,action='read')
+    atomicdatafile = trim(PREFIX)//'/share/neat/'//trim(atomicdatafile)//'.dat'
+    open(unit=1, status = 'old', file=atomicdatafile,action='read')
 
 !read # of comment lines and skip them
         read(1,*)NCOMS
